import express from 'express';
import swaggerUi from 'swagger-ui-express';
import YAML from 'yamljs';
import { animals, requests } from './resources';

/**
 * When in doubt check documentation
 * @see https://www.prisma.io/docs/concepts/overview/prisma-in-your-stack/rest
 *
 * Naming of the resources is inspired by:
 * @see https://laravel.com/docs/9.x/controllers#actions-handled-by-resource-controller
 *
 * @help Feeling stuck? Just check requests for examples
 */
const api = express();
const swaggerDocument = YAML.load(`${__dirname}/../docs/swagger.yaml`);

/**
 * Show Swagger UI for API Documentation
 */
api.use('/api-docs', swaggerUi.serve, swaggerUi.setup(swaggerDocument));

/**
 * Make express parse JSON in body and parse url encoded strings
 */
api.use(express.json());
api.use(express.urlencoded({ extended: true }));

/**
 * Serve static content from public directory for images
 */
api.use(express.static('public'));

/**
 * Send greetings to API Client
 */
api.get('/', (_req, res) => res.send({
  status: 'success',
  data: {},
  message: 'Welcome to our API',
}));

/**
 * Resource animals
 */
api.get('/animals', animals.list);
api.post('/animals', animals.store);
api.get('/animals/:id', animals.get);

/**
 * Resource requests
 */
api.get('/requests', requests.list);
api.get('/requests/:id', requests.show);
api.patch('/requests/:id', requests.partialUpdate);
api.put('/requests/:id', requests.update);
api.post('/requests', requests.store);
api.delete('/requests/:id', requests.destroy);

/**
 * Start listening on connections
 */
api.listen(process.env['PORT'], () => console.log(`Example app listening on port ${process.env['PORT']}`));
